<?php
require_once("common.php");
require_once("Video.class.php");
require_once("pid.class.php");

/***************************************************
Runcache class.
Jeff C
This class contains the methods that are used in 
the background tasks that are run by Wordpress cron
They are called from runcache.php
***************************************************/
class Runcache
{
	public $download_fails=0;	
	
	
	function __destruct()
	{	
		$this->delete_temp_options();
	}
	
	// --------------------------------------------------------------------------
	function delete_temp_options()
	{
		delete_option('dc_total_feeds');
		delete_option('dc_total_entries');
		delete_option('dc_current_feed_number');
		delete_option('dc_current_entry_number');
		delete_option('dc_current_feed');
		delete_option('dc_current_entry');
	}
	
	// --------------------------------------------------------------------------
	function ping_deletecity_com()
	{
		if(get_option('dc_share')!='1') 
		{
			print "\t Sharing not on.  Skipping step\n";
			return;
		}
		
		// Communicate with central DeleteCity API
		$url = "http://www.deletecity.com/api";
		print("\tPinging $url with blog_name, blog_url\n");
		$data = array(
				'blog_name' => get_bloginfo('name'), 
				'blog_url' 	=> get_bloginfo('url') );
		$params = array('http' => array(
			'header' => 'User-Agent: deletecitybot\r\n',
			'method' => 'POST',
			'content' => http_build_query($data)));
		$ctx = stream_context_create($params);
		$fp = @fopen($url, 'rb', false, $ctx);
		if (!$fp) 
		{
			print("\tProblem with $url, $php_errormsg\n");
			return;
		}
		$response = @stream_get_contents($fp);
		if ($response === false)
		{
			print "\tProblem reading data from $url, $php_errormsg\n";
			return;
		}
	}
	
	//----------------------------------------------------
	// Checks the cache_dir to make sure that every video in there
	// has a corresponding record in the database.
	// They could get out of sync if the database table is deleted or something.
	function check_for_orphans()
	{		
		if(!get_option('dc_cache_dir'))
			throw new Exception("Can't find dc_cache_dir");
		
		$dhandle = opendir(get_option('dc_cache_dir'));
		if(!$dhandle)
			throw new Exception("Couldn't open cache directory");

		// Make an array of all of the mp4 file names
		$files = array();
		while (false !== ($fname = readdir($dhandle)))
		{
			$ext = substr(strrchr($fname, '.'), 1);
			if($ext == "mp4") $files[] = $fname;
		}
		closedir($dhandle);
		
		
		// Loop through all of the mp4 files in the cache directory
		$num_files = count($files);
		for($i=0; $i<$num_files; $i++)
		{
			$path_parts = pathinfo($files[$i]);
			$youtube_id = $path_parts['filename'];
			if(empty($youtube_id))
				continue;

			if(!Video::exists( $youtube_id ))
			{
				$data_url = "http://gdata.youtube.com/feeds/api/videos/{$youtube_id}";
				print "\t[$i/$num_files] Found an orphaned video: {$youtube_id}\n";
				$response = Video::get_web_page( $data_url );
				if($response['errno'] > 0)
				{
					print "\t[$i/$num_files] Can't fetch info for {$youtube_id}!  Was it removed???\n";
					continue;
				}
				
				if($response['content']=="Invalid id")
				{
					print "\t[$i/$num_files] {$youtube_id} Invalid id!\n";
					continue;
				}
			
				if($response['content']=="Private video")
				{
					print "\t[$i/$num_files] {$youtube_id} Video has been marked as private!\n";
					continue;
				}
				if($response['content']=='Video not found')
				{
					print "\t[$i/$num_files] {$youtube_id} Video not found!\n";
					continue;
				}
			
				try {
				
					libxml_use_internal_errors(true);
					$xmldoc = new SimpleXMLElement( $response['content'] );
			
					if(!isset($xmldoc->title) || !isset($xmldoc->author->name) || !isset($xmldoc->link[0]['href']))
						throw new Exception("Not a valid XML file");
			
					Video::create($xmldoc->link[0]['href'], $xmldoc->title, $xmldoc->description, $xmldoc->author->name);
					print "\t[$i/$num_files] Created video {$youtube_id}, \"$xmldoc->title\"\n";

					
				} catch(Exception $e) {
					
					print "\t[$i/$num_files] {$youtube_id} ".$e->getMessage()."\n";
					foreach (libxml_get_errors() as $error) {
        				print "\t\t{$error}";
  					}
				}
			}
		}
	}
	
	
	//----------------------------------------------------
	// This is the main caching process.
	function cache()
	{
		if(!get_option('dc_cache_dir'))
			throw new Exception("Can't find dc_cache_dir");
		
		// Get the full feed URLs from the dc_sources option
		$feeds = get_option('dc_sources');
		
		// Now make additional feeds based on the keywords
		$keywords = explode(",", get_option("dc_keywords"));
		foreach($keywords as $keyword)
		{
			$keyword = trim($keyword);
			$keyword = urlencode($keyword);
			$feeds[] = "http://gdata.youtube.com/feeds/api/videos?q={$keyword}&orderby=published&max-results=50";
		}
		
		add_option('dc_total_feeds', '');
		add_option('dc_current_feed_number', '');
		add_option('dc_current_feed', '');
		add_option('dc_total_entries', '');
		add_option('dc_current_entry_number', '');
		add_option('dc_current_entry', '');
		
		// Loop through all of the feeds
		shuffle($feeds);
		$this->download_failures=0;
		$num_feeds = count($feeds);
		update_option('dc_total_feeds', $num_feeds);
		for($i=0; $i<$num_feeds; $i++)
		{
			printf("\n\tFeed [%d/%d] %s\n", $i+1, $num_feeds, $feeds[$i]);
			update_option('dc_current_feed_number', $i);
			update_option('dc_current_feed', $feeds[$i]);
			$this->process_feed( $feeds[$i] );
		}
		

		return true;
	}
	
	
	//----------------------------------------------------
	// Deals with all of the videos in a single gdata feed.
	function process_feed( $feed_url )
	{
		// Download the feed
		$response = Runcache::get_web_page( $feed_url );
		if($response['errno']>0)
		{
			print "\tError: {$response['errmsg']}\n";
			return;
		}
		
		// Try to parse feed
		$xmldoc = simplexml_load_string($response['content']);
		if (!$xmldoc)
		{
			$errors = libxml_get_errors();
			foreach ($errors as $error)
				echo display_xml_error($error, $xml);
			libxml_clear_errors();
			return;
		}
	
		if(!isset($xmldoc->entry) || count($xmldoc->entry)==0)
		{
			print "\tError: Feed doesn't contain any entries\n";
			return;
		}
		
	
		// Loop through all of the "entries" in the feed.
		$num_entries = count($xmldoc->entry);
		update_option('dc_total_entries', $num_entries);
		for($i=0; $i<$num_entries; $i++)
		{	
			$ar = getDirectorySize(get_option('dc_cache_dir'), "mp4"); 
			$megs = $ar['size'] / 1048576;
			if($megs > get_option('dc_max_cache_size'))
				throw new Exception("Cache exceeded max_cache_size.  Exiting caching process until some videos expire.");
		
			$entry = $xmldoc->entry[$i];
			$link = $entry->link[0]['href'];
			printf("\t\tEntry [%d/%d] %s\n", $i+1, $num_entries, $link);
			update_option('dc_current_entry_number', $i);
			update_option('dc_current_entry', $entry->title);
			$this->process_entry($feed_url, $link, $entry->title, $entry->content, $entry->author->name);
		} 
	}
	
	
	
	//----------------------------------------------------
	function process_entry($feed_url, $vid_url, $title, $content, $author)
	{
		// Find the $youtube_id of the video in the link element
		// TO DO:  We can't be sure that the href is element 0
		preg_match("/v=([^&]+)/", $vid_url, $matches);
		if( count($matches) < 2 )
		{
			print "\t\t\t couldn't find 'v=' in {$vid_url}";
			return;
		}
		$youtube_id = $matches[1];
		
		
		// Skip any videos that contain a blacklisted word.
		if($words = get_blacklisted_words($title, $content, $author)) 
		{
			$words = implode(", ", $words);
			print "\t\t\t\"$words\" found in video \"{$title}\". Skipping\n";
		}
		else if($video = Video::loadFromYouTubeID( $youtube_id ))
		{
			print "\t\t\tResetting expiration countdown for {$video->youtube_id}\n";
			$video->seen_in_feed();
		}
		else if($video = Video::create($vid_url, $title, $content, $author, $feed_url))
		{
			print "\t\t\tCreated video {$video->youtube_id}, \"{$video->title}\"\n";
		}
		else
		{
			$this->download_failures++;
			print "\t\t\t Download failed for {$vid_url}.  Total failures: {$this->download_failures}\n";
			if($this->download_failures>9)
				throw new Exception("Too many failures.  Exiting now for cooloff.");
		}
	}
	
	//----------------------------------------------------
	// loop through every video where status='cached' and check to see if it still exists on YouTube
	function check_for_deleted()
	{
		global $wpdb;
		
		if(!get_option('dc_max_age')) 
			throw new Exception("Can't find dc_max_age");
		
		$table = get_option('dc_videos_table');
		$sql = "SELECT youtube_id FROM $table WHERE status='cached'";
		
		$result = $wpdb->get_results($sql, ARRAY_A); 
		if($result===false)
			throw new Exception("\tProblem fetching cached videos");

		$total = count($result);
		$i=1;
		foreach($result as $row)
		{
			$video = Video::loadFromYouTubeID( $row['youtube_id'] );
			if($video->remote_still_exists())
			{
				$time = $video->age;
				$seconds = $time % 60;
				$mins = floor($time/60) % 60;
				$hours = floor($time/60/60) % 24;
				$days = floor($time/60/60/24);
				
				$parts = array();
				if($days) $parts[] = 	"{$days} days";
				if($hours) $parts[] = 	"{$hours} hours";
				if($mins) $parts[] = 	"{$mins} minutes";
				if($seconds) $parts[] = "{$seconds} seconds";
				$interval = implode(", ", $parts);
			
				print "\t[$i/$total] {$row['youtube_id']} still exists.  Age={$interval}\n";
				if( $days > get_option('dc_max_age') )
				{
					print "\t[$i/$total] {$row['youtube_id']} has expired.  Deleting video file.\n";
					$video->expire();
				}
			}
			else
			{
				print "\t[$i/$total] {$row['youtube_id']} has been removed!\n";
				$video->mark_as_removed();
			} 
			$i++;
		}
	}
	

	
	// ------------------------------------
	static function get_web_page( $url )
	{
		$ch      = curl_init( $url );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_HEADER, false );
		curl_setopt( $ch, CURLOPT_ENCODING, "" );
		curl_setopt( $ch, CURLOPT_USERAGENT, "spider" );
		curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 120 );
		curl_setopt( $ch, CURLOPT_TIMEOUT, 120 );
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
		$content = curl_exec( $ch );
		$err     = curl_errno( $ch );
		$errmsg  = curl_error( $ch );
		$header  = curl_getinfo( $ch );
		curl_close( $ch );
	
		$header['errno']   = $err;
		$header['errmsg']  = $errmsg;
		$header['content'] = $content;
		return $header;
	}	
}

?>